import base64
import cryptography

from lxml import etree
from signxml import XMLVerifier

from django.conf import settings
from django.shortcuts import redirect
from django.views.generic import View
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from django.contrib.auth.decorators import login_required

from django.utils.encoding import smart_str


saml_confs = getattr(settings, 'SAML', '')


@method_decorator(login_required, name="dispatch")
@method_decorator(csrf_exempt, name="dispatch")
class ProcessSAMLResponse(View):
    success = False

    def post(self, request, *args, **kwargs):
        # Read session ID from relay state data
        sessid = request.POST.get("RelayState", "")

        # Sanity check for the Session ID
        # Get the URL's from the session storage
        session_dict_key = saml_confs.get('session_dict', None)
        session_consumable = saml_confs.get('session_consume', None)
        succtarget = "/"
        errtarget = "/"

        # Check if key is already in the storage
        if session_dict_key in request.session:
            session_dict = request.session[session_dict_key]
            if session_dict is not None:
                succtarget = session_dict.get('success_url', succtarget)
                errtarget = session_dict.get('error_url', errtarget)

        # Read saml response
        bsamlresp = request.POST.get("SAMLResponse", "")
        samlresp = base64.b64decode(bsamlresp)
        signed_root = etree.fromstring(samlresp)

        # Verify data Signature
        cert_path = saml_confs.get('idp_cert_path')
        cert = open(cert_path).read()
        if(cert):

            try:
                # signed_xml
                assertion_data =\
                    XMLVerifier().\
                    verify(signed_root, x509_cert=cert).signed_xml
            except cryptography.exceptions.InvalidSignature:
                pass

            else:
                # Reset auth gov session data
                request.session[session_dict_key] = {}
                # Save session id from the auth request
                request.session[session_dict_key]['sessid'] = sessid

                # Read information from signed data
                samlns = "urn:oasis:names:tc:SAML:2.0:assertion"
                samlreqattributes = saml_confs.get('request_attributes')

                for attrid, attrname in samlreqattributes.items():
                    res = assertion_data.xpath(
                        '//fa:Attribute[@Name="{}"]'.format(attrname),
                        namespaces={'fa': samlns})
                    if (res and len(res) > 0):
                        attr = res[0]
                        if len(attr) > 0:
                            txt = attr[0].text
                            val = smart_str(txt)
                            request.session[session_dict_key][attrid] = val
                            self.success = True

        if(self.success):
            request.session[session_consumable] = True
            return redirect(succtarget)
        else:
            # Unset key if it exists
            request.session.pop(session_consumable, None)
            return redirect(errtarget)
