from __future__ import unicode_literals

from django.utils.translation import ugettext as _
from django.contrib.auth import get_user_model

from evotemessages.decorators import recipients
from evotemessages.recipients import BaseRecipient

from .models import Category


@recipients
class CategoryRecipient(BaseRecipient):
    """
    Group of voters of election category
    """
    prefix = 'election_category'
    model = Category
    label_prefix = _('Members of category')

    def get_recipients(self, *args, **kwargs):
        value = kwargs.get('q', '')
        slug = kwargs.get('slug', None)
        if slug:
            return self.get_queryset().filter(
                designation__icontains=value,
                election__electoral_process__slug__in=slug)
        else:
            return self.get_queryset().filter(
                election__identifier__icontains=value)

    def get_users(self, name):
        obj = self.get_object(name)
        if obj:
            users = obj.election_voters.values_list('user_id', flat=True)
            return get_user_model().objects.filter(id__in=users)
        return []

    def get_recipient_label(self, obj):
        return self.label_prefix + " " + obj.designation + " " +\
            _('of election') + " " + obj.election.identifier

    def check_permission(self, user):
        if hasattr(user, 'is_electoral_member'):
            return user.is_electoral_member
        return False
