from __future__ import unicode_literals

from django.core.files.base import ContentFile
from django.apps import apps

from evotevotings.voters.importers import ElectoralRollImporter
from evotedocs.models import Document
from celery.utils.log import get_task_logger
from evote.celery import app

from .documents import ElectoralRollPdf


logger = get_task_logger(__name__)


@app.task(ignore_result=True)
def import_electoral_roll(electoral_roll_id):
    """ """
    ElectoralRoll = apps.get_model("evotevoters.ElectoralRoll")  # noqa
    electoral_roll = ElectoralRoll.objects.select_related('election')\
        .get(pk=electoral_roll_id)
    try:
        importer = ElectoralRollImporter()
        importer.run(electoral_roll, electoral_roll.document.document)
    except Exception:
        logger.exception("Error importing electoral roll")
    finally:
        electoral_roll.imported = True
        electoral_roll.save()


@app.task(ignore_result=True)
def generate_electoral_roll_pdf(electoral_roll_id):
    """ """
    ElectoralRoll = apps.get_model("evotevoters.ElectoralRoll")  # noqa
    electoral_roll = ElectoralRoll.objects.select_related('election')\
        .get(pk=electoral_roll_id)
    try:
        pdf = ElectoralRollPdf().render(electoral_roll=electoral_roll)
        file = Document(designation='electoral_roll')
        file_name = electoral_roll.election.identifier + "_" +\
            electoral_roll.version + ".pdf"
        file.document.save(file_name, ContentFile(pdf), False)
        file.save()
        electoral_roll.pdf_document = file
        electoral_roll.save()
    except Exception:
        logger.exception("Error generating electoral roll pdf")
